/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */

package org.objectweb.jorm.runtime.mapper.rdb.adapter.api;

import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.type.api.PType;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.math.BigDecimal;

/*
 * This interface permits to do the conversion between the jorm types PType and
 * the sql types. It is also possible to fetch the function name of getter and
 * setter methods associated to a PType.
 *
 * @author : S.Chassande-Barrioz
 */

public interface RdbAdpater {
	/**
	 * This methods return the SQL type linked to the PType
	 * specified in parameter.
	 */
	String getSqlType(PType pt, boolean usedInPK) throws PExceptionProtocol;

	/**
	 * This methods return the sql getter method associated to the PType
	 * specified in parameter. The returned string is the fetching of the
     * value on the resultset to the specified index:
     * "@resultSet.getBoolean(@idx)"
	 */
	String getSqlGetterName(PType pt, String resultSet, int idx)
        throws PExceptionProtocol;

	/**
	 * This methods return the the sql setter method associated to the PType
	 * specified in parameter. The returned string is the assignment of the
     * value to the specified index: "setBoolean(@idx, @value)"
	 */
	String getSqlSetterName(PType pt, String value, int idx)
        throws PExceptionProtocol;

	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	int getSqlTypeCode(PType pt) throws PExceptionProtocol;

	/**
	 * This calculates the size of a result set. If this feature is not supported
	 * by a database or its jdbc driver, this method returns -1.
	 * @ResultSet rs the result set which the size must be returned
	 */
	int fetchResultSetSize(ResultSet rs) throws SQLException;

    boolean getBoolean(ResultSet rs, int idx) throws SQLException;
    Boolean getOboolean(ResultSet rs, int idx) throws SQLException;
    char getChar(ResultSet rs, int idx) throws SQLException;
    Character getOchar(ResultSet rs, int idx) throws SQLException;
    byte getByte(ResultSet rs, int idx) throws SQLException;
    Byte getObyte(ResultSet rs, int idx) throws SQLException;
    short getShort(ResultSet rs, int idx) throws SQLException;
    Short getOshort(ResultSet rs, int idx) throws SQLException;
    int getInt(ResultSet rs, int idx) throws SQLException;
    Integer getOint(ResultSet rs, int idx) throws SQLException;
    long getLong(ResultSet rs, int idx) throws SQLException;
    Long getOlong(ResultSet rs, int idx) throws SQLException;
    float getFloat(ResultSet rs, int idx) throws SQLException;
    Float getOfloat(ResultSet rs, int idx) throws SQLException;
    double getDouble(ResultSet rs, int idx) throws SQLException;
    Double getOdouble(ResultSet rs, int idx) throws SQLException;
    String getString(ResultSet rs, int idx) throws SQLException;
    Date getDate(ResultSet rs, int idx) throws SQLException;
	BigDecimal getBigDecimal(ResultSet rs, int idx) throws SQLException;
    char[] getCharArray(ResultSet rs, int idx) throws SQLException;
    byte[] getByteArray(ResultSet rs, int idx) throws SQLException;
    Object getObject(ResultSet rs, int idx) throws SQLException;
}
