/**
 * copyright area
 */
package org.objectweb.jorm.util.rdb;

import java.io.PrintStream;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

/**
 * This tools clas permits to check the conformance of a jdbc driver to the
 * standard. The -interac option is an interactive mode that permits to specify
 * the SQL type, the java type and optionaly the jdbc accessors.
 *
 * @author S.Chassande-Barrioz
 */
public class DriverConformTest {

	/**
	 * Main method to launch the tests manually.
	 */
	public static void main(String[] args) {
		boolean interac = false;
		boolean debug = false;
		out = System.out;
		String sql = null;
		String javaType = null;
		String value = null;
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-driver") && (i + 1) < args.length)
				drvCN = args[++i];
			else if (args[i].equals("-url") && (i + 1) < args.length)
				url = args[++i];
			else if (args[i].equals("-user") && (i + 1) < args.length)
				user = args[++i];
			else if (args[i].equals("-passwd") && (i + 1) < args.length)
				passwd = args[++i];
			else if (args[i].equals("-sql") && (i + 1) < args.length)
				sql = args[++i];
			else if (args[i].equals("-java") && (i + 1) < args.length)
				javaType = args[++i];
			else if (args[i].equals("-value") && (i + 1) < args.length)
				value = args[++i];
			else if (args[i].equals("-debug"))
				debug = true;
			else {
				System.out.println("Unknown argument: " + args[i]);
				usage();
				System.exit(1);
			}

		}
		DriverConformTest d = new DriverConformTest(debug);
		if (interac) {
			String getter = null;
			String setter = null;
			try {
				sql = readString("Enter a SQL TYPE: ");
				javaType = readString("Enter the java type (ex: java.lang.Integer): ");
				value = readString("Enter a value(ex: 0): ");
				String answer = readString("Do you wan to use standard sql accessors [Y|n]:");
				getter = null;
				setter = null;
				if ("n".equalsIgnoreCase(answer.trim())) {
					getter = readString("Enter the sql getter (ex: getBoolean): ");
					setter = readString("Enter the sql setter (ex: setBoolean): ");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			d.specif(sql, javaType, value, getter, setter);
		} else if (sql != null && javaType != null && value != null) {
			d.specif(sql, javaType, value, null, null);
		} else {
			d.standard();
		}
		d.close();
	}

	private static void usage() {
		System.out.println("\nUsage: DriverConformTest"
			+ " -driver <driver class name> "
			+ " -url <database url> "
			+ "\n [-user <user name> -passwd <password of the user>]"
			+ " -debug"
			+ "\n -sql <SQL type> "
			+ " -java <java type> "
			+ " -value <a value> "
		);
		System.out.println("* debug: The debug option prints all test results.");
		//System.out.println("* interac: This mode permits to test a non standard SQL type. This mode is interactive.");
	}

	private static String drvCN;
	private static String user;
	private static String url;
	private static String passwd;
	private static PrintStream out;

	private Connection conn = null;
	private boolean debug;

	public DriverConformTest(boolean debug) {
		this.debug = debug;
		if (drvCN == null || drvCN.length() == 0) {
			System.out.println("No driver specified !");
			usage();
			System.exit(1);
		}
		try {
			//Load the driver
			Class.forName(drvCN);
		} catch (Exception e) {
			out.println("Impossible to load the driver: " + drvCN);
			out.println(e.getMessage());
			System.exit(1);
		}
		if (url == null || url.length() == 0) {
			out.println("No url specified !");
			usage();
			System.exit(1);
		}
		//Fetch a connection
		try {
			if (user == null || user.length() == 0)
				conn = DriverManager.getConnection(url);
			else
				conn = DriverManager.getConnection(url, user, passwd);
		} catch (SQLException e) {
			out.println("Impossible to fetch a connection:");
			out.println(e.getMessage());
			System.exit(1);
		}
	}

	public void specif(String sqlType, String stype, String sval, String getter, String setter) {
		try {
			Object val = null;
			Class type = null;
			try {
				type = Class.forName(stype);
				val = type
					.getDeclaredMethod("valueOf", new Class[]{String.class})
					.invoke(null, new Object[]{sval});
			} catch (Exception e) {
				try {
					val = type
						.getDeclaredMethod("valueOf", new Class[]{Object.class})
						.invoke(null, new Object[]{sval});
				} catch (Exception e2) {
					System.err.println("Impossible to build a " + sval);
					System.exit(1);
				}
			}
			String tableName = "table_" + sqlType;
			_testCreateTable(tableName, sqlType);
			_testInsertSelect(sqlType, tableName, val, type, getter, setter);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void standard() {
		testBoolean();
		testByte();
		testShort();
		testInt();
		testLong();
		testFloat();
		testDouble();
		testDate();
		testString();
		testBigDecimal();
	}

	public void close() {
		if (conn != null)
			try {
				conn.close();
			} catch (SQLException e) {
			}
	}

	private boolean _testCreateTable(String tableName, String colType) {
		Statement s = null;
		try {
			s = conn.createStatement();
			s.execute("DROP TABLE " + tableName);

		} catch (SQLException e) {
		} finally {
			try {
				s.close();
			} catch (SQLException e) {
			}
		}
		try {
			s = conn.createStatement();
			s.execute(
				"CREATE TABLE " + tableName + "(col1 " + colType + ")");
			ok("create table", colType);
			return true;
		} catch (SQLException e) {
			fail("create table: " + e.getMessage(), colType);
			return false;
		} finally {
			try {
				s.close();
			} catch (SQLException e) {
			}
		}
	}

	private void _testInsertSelect(String colType,
								   String tableName,
								   Object val,
								   Class type,
								   String getter,
								   String setter) {
		String sval = "" + val;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			ps = conn.prepareStatement("INSERT INTO " + tableName + " VALUES (?)");
			assignValue(ps, 1, val, type, setter);
			ps.execute();
			ok("insert value " + sval, colType);

			ps = conn.prepareStatement("SELECT * from " + tableName + " WHERE col1 = ?");
			assignValue(ps, 1, val, type, setter);
			rs = ps.executeQuery();
			if (rs.next()) {
				if (val.equals(retrieveValue(rs, 1, type, getter)))
					ok("select value " + sval, colType);
				else
					fail("select value " + sval + ": bad value", colType);

				Object o = rs.getObject(1);
				if (type.equals(o.getClass()) &&
					((val == null && o == null) || (val != null && val.equals(o))))
					ok("select value " + sval + " with getObject", colType);
				else
					fail("select value " + sval + " with getObject", colType);

				if (rs.next())
					fail("select value " + sval + ": more than one result", colType);
			} else
				fail("select value " + sval + ": no result", colType);
		} catch (Exception e) {
			fail("insert & select: ", e, colType);
		} finally {
			try {
				if (rs != null)
					rs.close();
			} catch (SQLException e) {
			}
			try {
				if (ps != null)
					ps.close();
			} catch (SQLException e) {
			}
		}
	}

	private void assignValue(PreparedStatement ps,
							 int index,
							 Object value,
							 Class type,
							 String setter) throws Exception {
		if (value == null) {
			ps.setNull(index, class2sqlType(type));
		} else if (setter != null && setter.length() > 0) {
			try {
				PreparedStatement.class.getMethod(setter, new Class[]{Integer.TYPE, type})
					.invoke(ps, new Object[]{new Integer(1), value});
			} catch (NoSuchMethodException e) {
				fail("Method " + setter + " not found");
				throw e;
			}
		} else if (Boolean.class.equals(type))
			ps.setBoolean(index, ((Boolean) value).booleanValue());

		else if (Byte.class.equals(type))
			ps.setByte(index, ((Byte) value).byteValue());

		else if (Short.class.equals(type))
			ps.setShort(index, ((Short) value).shortValue());

		else if (Integer.class.equals(type))
			ps.setInt(index, ((Integer) value).intValue());

		else if (Long.class.equals(type))
			ps.setLong(index, ((Long) value).longValue());

		else if (Float.class.equals(type))
			ps.setFloat(index, ((Float) value).floatValue());

		else if (Double.class.equals(type))
			ps.setDouble(index, ((Double) value).doubleValue());

		else if (String.class.equals(type))
			ps.setString(index, (String) value);

		else if (Date.class.equals(type))
			ps.setDate(index, (Date) value);

		else if (BigDecimal.class.equals(type))
			ps.setBigDecimal(index, (BigDecimal) value);

		else
			throw new Exception("Unmanaged type: " + type);
	}

	private int class2sqlType(Class type) throws Exception {
		if (Boolean.class.equals(type))
			return Types.BIT;
		else if (Byte.class.equals(type))
			return Types.TINYINT;
		else if (Short.class.equals(type))
			return Types.SMALLINT;
		else if (Integer.class.equals(type))
			return Types.INTEGER;
		else if (Long.class.equals(type))
			return Types.BIGINT;
		else if (Float.class.equals(type))
			return Types.REAL;
		else if (Double.class.equals(type))
			return Types.DOUBLE;
		else if (String.class.equals(type))
			return Types.VARCHAR;
		else if (Date.class.equals(type))
			return Types.DATE;
		else if (BigDecimal.class.equals(type))
			return Types.DECIMAL;
		else
			throw new Exception("Unmanaged type: " + type);
	}

	private Object retrieveValue(ResultSet rs,
								 int index,
								 Class type,
								 String getter) throws Exception {
		Object o = null;
		if (getter != null && getter.length() > 0) {
			try {
				o = ResultSet.class.getMethod(getter, new Class[]{Integer.TYPE})
					.invoke(rs, new Object[]{new Integer(1)});
			} catch (NoSuchMethodException e) {
				fail("Method " + getter + " not found");
				throw e;
			}
		} else if (Boolean.class.equals(type))
			o = new Boolean(rs.getBoolean(index));

		else if (Byte.class.equals(type))
			o = new Byte(rs.getByte(index));

		else if (Short.class.equals(type))
			o = new Short(rs.getShort(index));

		else if (Integer.class.equals(type))
			o = new Integer(rs.getInt(index));

		else if (Long.class.equals(type))
			o = new Long(rs.getLong(index));

		else if (Float.class.equals(type))
			o = new Float(rs.getFloat(index));

		else if (Double.class.equals(type))
			o = new Double(rs.getDouble(index));

		else if (String.class.equals(type))
			o = rs.getString(index);

		else if (Date.class.equals(type))
			o = rs.getDate(index);

		else if (BigDecimal.class.equals(type))
			o = rs.getBigDecimal(index);

		else
			throw new Exception("Unknown type: " + type);
		return (rs.wasNull() ? null : o);

	}


	public void testBoolean() {
		String tableName = "JormBoolean";
		String colType = "BIT";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, Boolean.TRUE, Boolean.class, null, null);
			_testInsertSelect(colType, tableName, Boolean.FALSE, Boolean.class, null, null);
		}
	}

	public void testByte() {
		String tableName = "JormByte";
		String colType = "TINYINT";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Byte((byte) 0), Byte.class, null, null);
			_testInsertSelect(colType, tableName, new Byte((byte) 1), Byte.class, null, null);
			_testInsertSelect(colType, tableName, new Byte((byte) 254), Byte.class, null, null);
		}
	}

	public void testShort() {
		String tableName = "JormShort";
		String colType = "SMALLINT";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Short((short) 0), Short.class, null, null);
			_testInsertSelect(colType, tableName, new Short((short) 1), Short.class, null, null);
			_testInsertSelect(colType, tableName, new Short((short) 254), Short.class, null, null);
		}
	}


	public void testInt() {
		String tableName = "JormInt";
		String colType = "INTEGER";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Integer(0), Integer.class, null, null);
			_testInsertSelect(colType, tableName, new Integer(1), Integer.class, null, null);
			_testInsertSelect(colType, tableName, new Integer(254), Integer.class, null, null);
		}
	}

	public void testLong() {
		String tableName = "JormLong";
		String colType = "BIGINT";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Long(0), Long.class, null, null);
			_testInsertSelect(colType, tableName, new Long(1), Long.class, null, null);
			_testInsertSelect(colType, tableName, new Long(254), Long.class, null, null);
		}
	}

	public void testFloat() {
		String tableName = "JormFloat";
		String colType = "REAL";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Float(0.0), Float.class, null, null);
			_testInsertSelect(colType, tableName, new Float(1), Float.class, null, null);
			_testInsertSelect(colType, tableName, new Float(254), Float.class, null, null);
		}
	}

	public void testDouble() {
		String tableName = "JormDouble";
		String colType = "DOUBLE";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new Double(0.0), Double.class, null, null);
			_testInsertSelect(colType, tableName, new Double(1), Double.class, null, null);
			_testInsertSelect(colType, tableName, new Double(254), Double.class, null, null);
		}
	}

	public void testDate() {
		String tableName = "JormDate";
		String colType = "DATE";
		if (_testCreateTable(tableName, colType)) {
			//_testInsertSelect(colType, tableName, new Date(), Double.class, null, null);
		}
	}

	public void testString() {
		_testCreateTable("JormString", "VARCHAR");
		String tableName = "JormString";
		String colType = "VARCHAR";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, "", String.class, null, null);
			_testInsertSelect(colType, tableName, "abcd", String.class, null, null);
		}
	}

	public void testBigDecimal() {
		_testCreateTable("JormBigDecimal", "DECIMAL");
		String tableName = "JormBigDecimal";
		String colType = "DECIMAL";
		if (_testCreateTable(tableName, colType)) {
			_testInsertSelect(colType, tableName, new BigDecimal((double) 0.0), BigDecimal.class, null, null);
			_testInsertSelect(colType, tableName, new BigDecimal("8764236557855635636.78966564"), BigDecimal.class, null, null);
		}
	}

	public static String readString(String invitation) throws Exception {
		System.out.print(invitation);
		System.out.flush();
		StringBuffer reply_buffer = null;
		char b = (char) System.in.read();
		while (b > 0 && b != '\n' && Character.isWhitespace(b)) {
			b = (char) System.in.read();
		}
		while (b > 0 && b != '\n' && b != 13) {
			if (reply_buffer == null) {
				reply_buffer = new StringBuffer();
			}
			reply_buffer.append(b);
			b = (char) System.in.read();
		}
		if (System.in.available() == 1) {
			System.in.read();
		}
		if (reply_buffer != null) {
			return reply_buffer.toString();
		} else {
			return "";
		}
	}

	public void fail(String msg, Throwable t, String colType) {
		if (debug) {
			fail(colType + ": " + msg);
			t.printStackTrace(out);
		} else
			fail(colType + ": " + msg + t.getMessage());
	}

	public void fail(String msg, String colType) {
		fail(colType + ": " + msg);
	}

	public void fail(String msg) {
		out.println("**FAIL: " + msg);
	}

	public void ok(String msg, String colType) {
		ok(colType + ": " + msg);
	}

	public void ok(String msg) {
		if (debug)
			out.println("OK: " + msg);
	}

	public void debug(String msg) {
		if (debug)
			out.println("DEBUG: " + msg);
	}
}
